/**
 *  Api匹配器
 *  Created by LooooG on 2018/04/11.
 *
 * (所有Api回调函数：success，fail，complete暂不处理)
 */

/**
 * 兼容方式 - 接口
 * 对于新增的 API，可以用以下代码来判断是否支持用户的手机。
 */
let showVersionTips = function() {
    // 如果希望用户在最新版本的客户端上体验您的小程序，可以这样子提示
    wx.showModal({
        title: '提示',
        content: '当前微信版本过低，无法使用该功能，请升级到最新微信版本后重试。'
    })
    return undefined
}

export default {
    getSystemInfo() {
        let info = null
        wx.getSystemInfo({
            success: res => {
                info = res
            }
        })
        console.log(info)
        return info
    },
    /****** 网络 ******/

    /* 上传、下载 */
    /**
     * 上传
     *
     * @param {String} url 开发者服务器 url
     * @param {String} filePath 要上传文件资源的路径
     * @param {String} name 文件对应的 key , 开发者在服务器端通过这个 key 可以获取到文件二进制内容
     * @param {Object} formData HTTP 请求中其他额外的 form data
     * @param {Function} cbOk 接口调用成功的回调函数
     * @param {Function} cbErr 接口调用失败的回调函数
     * @returns {uploadTask}
     * 返回一个 uploadTask 对象，通过 uploadTask，
     * 可监听上传进度变化事件:
     * uploadTask.onProgressUpdate((res) => {
     *      console.log('上传进度', res.progress)
     *      console.log('已经上传的数据长度', res.totalBytesSent)
     *      console.log('预期需要上传的数据总长度', res.totalBytesExpectedToSend)
     *  })
     * 以及取消上传任务:
     * uploadTask.abort()
     */
    uploadFile(url, filePath, name, formData, cbOk, cbErr) {
        return wx.uploadFile({
            url: url,
            filePath: filePath,
            name: name,
            formData: formData,
            success(res) {
                if (!res || res.data.indexOf('<Error>') > -1) {
                    console.log('上传失败', res)
                    cbErr(res)
                } else {
                    console.log('上传成功', res)
                    cbOk(res)
                }
            },
            fail(res) {
                console.log('上传失败', res)
                cbErr(res)
            }
        })
    },

    /****** 媒体 ******/

    /* 图片 */
    /**
     * ​从本地相册选择图片或使用相机拍照
     *
     * @param {Number} [count] 最多可以选择的图片张数，默认1
     * @param {Array} [sizeType] 默认['original', 'compressed'], original 原图，compressed 压缩图，默认二者都有
     *  @param {Array} [sourceType] 默认['album', 'camera'],original 原图，album 从相册选图，camera 使用相机，默认二者都有
     * @param {Function} [successCallback] 成功则返回图片的本地文件路径列表的回调
     * @param {Function} [failCallback] 接口调用失败的回调函数
     * @param {Function} [completeCallback] 接口调用结束的回调函数（调用成功、失败都会执行）
     */
    chooseImage(count, sizeType, sourceType, successCallback, failCallback, completeCallback) {
        wx.chooseImage({
            count: count || 1,
            sizeType: sizeType || ['original', 'compressed'],
            sourceType: sourceType || ['album', 'camera'],
            success: function (res) {
                typeof successCallback === 'function' && successCallback(res)
            },
            fail: function (res) {
                typeof failCallback === 'function' && failCallback(res)
            },
            complete: function (res) {
                typeof completeCallback === 'function' && completeCallback(res)
            }
        })
    },

    /**
     * 预览图片
     *
     * @param {String} currentUrl 当前显示图片的链接，不填则默认为 urls 的第一张
     * @param {Array} urls 需要预览的图片链接列表
     */
    previewImage(currentUrl, urls) {
        wx.previewImage({
            current: currentUrl,
            urls: urls
        })
    },

    /**
     * 获取图片信息
     *
     * @param {String} src 图片的路径，可以是相对路径，临时文件路径，存储文件路径，网络图片路径
     * @param {Function} cbOk 接口调用成功的回调函数
     * @returns {Promise.<*>}
     */
    getImageInfo(src, cbOk) {
        let that = this
        wx.getImageInfo({
            src: src,
            success(res) {
                cbOk(res)
            },
            fail() {
                that.showToast('获取图片信息失败')
            }
        })
    },

    /**
     * 保存图片到系统相册。需要用户授权 scope.writePhotosAlbum
     *
     * @param {String} filePath 图片的路径，可以是相对路径，临时文件路径，存储文件路径，网络图片路径
     */
    saveImageToPhotosAlbum(filePath) {
        let that = this
        if (wx.saveImageToPhotosAlbum) {
            wx.saveImageToPhotosAlbum({
                filePath: filePath,
                success(res) {
                },
                fail() {
                    that.showToast('保存图片到系统相册失败')
                }
            })
        } else {
            showVersionTips()
        }
    },

    /* 录音管理 */
    /**
     * 获取全局唯一的录音管理器 recorderManager
     *
     *  recorderManager.onStart(() => {
     *    console.log('recorder start')
     *  })
     *  recorderManager.onPause(() => {
     *    console.log('recorder pause')
     *  })
     *  recorderManager.onStop((res) => {
     *    console.log('recorder stop', res)
     *    const { tempFilePath } = res
     *  })
     *  recorderManager.onFrameRecorded((res) => {
     *    const { frameBuffer } = res
     *    console.log('frameBuffer.byteLength', frameBuffer.byteLength)
     *  })
     *
     *  const options = {
     *    duration: 10000,
     *    sampleRate: 44100,
     *    numberOfChannels: 1,
     *    encodeBitRate: 192000,
     *    format: 'aac',
     *    frameSize: 50
     *  }
     *
     * recorderManager.start(options)
     *
     * @returns {recorderManager}
     */
    getRecorderManager() {
        if (wx.getRecorderManager) {
            return wx.getRecorderManager()
        } else {
            return showVersionTips() // 微信低版本兼容提示
        }
    },

    /* 音频组件控制 */
    /**
     * 创建并返回内部 audio 上下文 innerAudioContext 对象。
     *
     * const innerAudioContext = wx.createInnerAudioContext()
     * innerAudioContext.autoplay = true
     * innerAudioContext.src = 'http://ws.stream.qqmusic.qq.com/M500001VfvsJ21xFqb.mp3?guid=ffffffff82def4af4b12b3cd9337d5e7&uin=346897220&vkey=6292F51E1E384E061FF02C31F716658E5C81F5594D561F2E88B854E81CAAB7806D5E4F103E55D33C16F3FAC506D1AB172DE8600B37E43FAD&fromtag=46'
     * innerAudioContext.onPlay(() => {
     *      console.log('开始播放')
     *  })
     * innerAudioContext.onError((res) => {
     *      console.log(res.errMsg)
     *      console.log(res.errCode)
     *  })
     *
     * @returns {innerAudioContext}
     */
    createInnerAudioContext() {
        if (wx.createInnerAudioContext) {
            return wx.createInnerAudioContext()
        } else {
            return showVersionTips() // 微信低版本兼容提示
        }
    },

    /****** 数据存储 ******/
    /**
     * 将 data 存储在本地缓存中指定的 key 中，会覆盖掉原来该 key 对应的内容，这是一个同步接口
     *
     * @param {String} key
     * @param {*} data
     */
    setStorageSync(key, data) {
        wx.setStorageSync(key, data)
    },

    /**
     * 从本地缓存中同步获取指定 key 对应的内容
     *
     * @param {String} key
     * @returns {*}
     */
    getStorageSync(key) {
        return wx.getStorageSync(key)
    },

    /**
     * 从本地缓存中同步移除指定 key
     *
     * @param {String} key
     */
    removeStorageSync(key) {
        wx.removeStorageSync(key)
    },

    /****** 设备 ******/

    /* 系统信息 */
    /**
     * 获取系统信息
     * brand    手机品牌    1.5.0
     model    手机型号
     pixelRatio    设备像素比
     screenWidth    屏幕宽度    1.1.0
     screenHeight    屏幕高度    1.1.0
     windowWidth    可使用窗口宽度
     windowHeight    可使用窗口高度
     statusBarHeight    状态栏的高度    1.9.0
     language    微信设置的语言
     version    微信版本号
     system    操作系统版本
     platform    客户端平台
     fontSizeSetting    用户字体大小设置。以“我-设置-通用-字体大小”中的设置为准，单位：px    1.5.0
     SDKVersion    客户端基础库版本
     *
     * @returns {Promise}
     */
    getSystemInfo() {
        return new Promise((resolve, reject) => {
            wx.getSystemInfo(res => {
                resolve(res)
            }, err => {
                reject(err)
            })
        })
    },

    /**
     * 获取系统信息, 同步接口
     *
     * @returns {Promise}
     */
    getSystemInfoSync() {
        return wx.getSystemInfoSync()
    },


    /****** 界面 ******/

    /* 交互反馈 */
    /**
     * 显示消息提示框
     *
     * @param {String} title 提示内容
     * @param {String} [icon] 图标，有效值 "success", "loading", "none"
     * @param {Number} [duration] 提示的延迟时间，单位毫秒，默认：1500
     * @param {Boolean} [mask] 是否显示透明蒙层，防止触摸穿透，默认：false
     * @param {String} [image] 自定义图标的本地路径，image 的优先级高于 icon
     */
    showToast(title, icon, duration, mask, image) {
        wx.showToast({
            title: title,
            icon: icon || 'none',
            duration: duration || 2000,
            mask: mask || false,
            image: image || ''
        })
    },

    /**
     * 隐藏消息提示框
     */
    hideToast() {
        wx.hideToast()
    },

    /**
     * 显示 loading 提示框, 需主动调用 wx.hideLoading 才能关闭提示框
     *
     * @param {String} title 提示内容
     * @param {Boolean} [mask] 是否显示透明蒙层，防止触摸穿透，默认：false
     */
    showLoading(title, mask) {
        wx.showLoading({
            title: title || '',
            mask: mask || false
        })
    },

    /**
     * 隐藏 loading 提示框
     */
    hideLoading() {
        wx.hideLoading()
    },

    /**
     * ​显示模态弹窗
     *
     * @param {String} content 提示的内容
     * @param {function} [callback]
     *      return --> confirm    Boolean    为 true 时，表示用户点击了确定按钮;
     *                 cancel    Boolean    为 true 时，表示用户点击了取消（用于 Android 系统区分点击蒙层关闭还是点击取消按钮关闭）
     * @param {String} [title] 提示的标题
     * @param {Boolean} [showCancel] 是否显示取消按钮，默认为 true - 本方法为false
     * @param {String} [cancelText] 取消按钮的文字，默认为"取消"，最多 4 个字符
     * @param {String} [cancelColor] 取消按钮的文字颜色，默认为"#000000"
     * @param {String} [confirmText] 确定按钮的文字，默认为"确定"，最多 4 个字符
     * @param {String} [confirmColor] 确定按钮的文字颜色，默认为"#3CC51F"
     */
    showModal(content, callback, title, showCancel, cancelText, cancelColor, confirmText, confirmColor) {
        wx.showModal({
            title: title || '提示',
            content: content || '未设置提示内容',
            showCancel: showCancel || false,
            cancelText: cancelText || '取消',
            cancelColor: cancelColor || '#666666',
            confirmText: confirmText || '确定',
            confirmColor: confirmColor || '#ff8c8d',
            success(res) {
                if (res.confirm) {
                    typeof callback === 'function' && callback(true)
                } else if (res.cancel) {
                    typeof callback === 'function' && callback(false)
                }
            }
        })
    },

    /* 设置导航条 */
    /**
     * 动态设置当前页面的标题。
     * @param title
     */
    setNavigationBarTitle(title) {
        wx.setNavigationBarTitle({
            title: title
        })
    },

    /**
     * 在当前页面显示导航条加载动画
     */
    showNavigationBarLoading() {
        wx.showNavigationBarLoading()
    },

    /**
     * 隐藏导航条加载动画
     */
    hideNavigationBarLoading() {
        wx.hideNavigationBarLoading()
    },

    /* 设置tabBar */
    /**
     * 为 tabBar 某一项的右上角添加文本
     *
     * @param {Number} index tabBar的哪一项，从左边算起, 第一项index=0
     * @param {String|Number} text 显示的文本，超过 3 个字符则显示成“…”
     */
    setTabBarBadge(index, text) {
        wx.setTabBarBadge({
            index: index,
            text: text
        }, succ => {
            console.log('setTabBarBadge succ', succ)
        }, err => {
            console.log('setTabBarBadge fail', err)
        }, res => {
            console.log('setTabBarBadge fin', ress)
        })
    },

    /**
     * 移除 tabBar 某一项右上角的文本
     *
     * @param {Number} index tabBar的哪一项，从左边算起, 第一项index=0
     */
    removeTabBarBadge(index) {
        wx.removeTabBarBadge({
            index: index
        })
    },


    /* 导航 */
    /**
     * 保留当前页面，跳转到应用内的某个页面，使用wx.navigateBack可以返回到原页面。
     *
     * @param url 需要跳转的应用内非 tabBar 的页面的路径 , 路径后可以带参数。
     * 参数与路径之间使用?分隔，参数键与参数值用=相连，不同参数用&分隔；如 'path?key=value&key2=value2'
     */
    navigateTo(url) {
        wx.navigateTo({
            url
        })
    },

    /**
     * 关闭当前页面，跳转到应用内的某个页面。
     *
     * @param url 需要跳转的应用内非 tabBar 的页面的路径 , 路径后可以带参数。
     * 参数与路径之间使用?分隔，参数键与参数值用=相连，不同参数用&分隔；如 'path?key=value&key2=value2'
     * (回调函数：success，fail，complete暂不使用)
     */
    redirectTo(url) {
        wx.redirectTo({
            url
        })
    },

    /**
     * 关闭当前页面，返回上一页面或多级页面。可通过 getCurrentPages()) 获取当前的页面栈，决定需要返回几层。
     *
     * @param {number} [delta] 返回的页面数，如果 delta 大于现有页面数，则返回到首页。
     */
    navigateBack(delta) {
        wx.navigateBack({
            delta: delta || 1
        })
    },

    /**
     * 跳转到 tabBar 页面，并关闭其他所有非 tabBar 页面
     * @param url 需要跳转的 tabBar 页面的路径（需在 app.json 的 tabBar 字段定义的页面），路径后不能带参数
     */
    switchTab(url) {
        wx.switchTab({
            url
        })
    },

    /* 绘图 */
    /**
     * 创建 canvas 绘图上下文（指定 canvasId）
     *
     * @param {String} canvasId 画布标识，传入定义在 <canvas/> 的 canvas-id
     * @param {Object} [componentInstance] 自定义组件实例 this ，表示在这个自定义组件下查找拥有 canvas-id 的 <canvas/> ，如果省略，则不在任何自定义组件内查找
     * @returns {*}
     */
    createCanvasContext(canvasId, componentInstance) {
        return wx.createCanvasContext(canvasId, componentInstance)
    },

    /**
     * 把当前画布指定区域的内容导出生成指定大小的图片，并返回文件路径。
     *
     * @param {String} canvasId 画布标识，传入 <canvas/> 的 canvas-id
     * @param {Function} cbOk 接口调用成功的回调函数
     * @param {Number} [x] 画布x轴起点（默认0）
     * @param {Number} [y] 画布y轴起点（默认0）
     * @param {Number} [width] 画布宽度（默认为canvas宽度-x）
     * @param {Number} [height] 画布高度（默认为canvas高度-y）
     * @param {Number} [destWidth] 输出图片宽度（默认为 width * 屏幕像素密度）
     * @param {Number} [desHeight] 输出图片高度（默认为 height * 屏幕像素密度）
     * @param {String} [fileType] 目标文件的类型，只支持 'jpg' 或 'png'。默认为 'png'
     * @param {Number} [quality] 图片的质量，取值范围为 (0, 1]，不在范围内时当作1.0处理
     * @param {Object} [componentInstance]
     * @returns {Promise.<*>}
     */
    canvasToTempFilePath(canvasId, cbOk, x, y, width, height, destWidth, desHeight, fileType, quality, componentInstance) {
        let tempFilePath
        wx.canvasToTempFilePath({
            canvasId: canvasId,
            x: x || 0,
            y: y || 0,
            width: width || undefined,
            height: height || undefined,
            destWidth: destWidth || undefined,
            destHeight: desHeight || undefined,
            fileType: fileType || 'png',
            quality: quality || 1.0,
            success(res) {
                cbOk(res.tempFilePath)
            }
        }, componentInstance)
        return tempFilePath
    },

    /* 下拉刷新 */
    /**
     * 开始下拉刷新，调用后触发下拉刷新动画，效果与用户手动下拉刷新一致
     * @returns {*}
     */
    startPullDownRefresh() {
        if (wx.startPullDownRefresh) {
            return wx.startPullDownRefresh()
        } else {
            return showVersionTips() // 微信低版本兼容提示
        }
    },

    /**
     * 停止当前页面下拉刷新
     * @returns {*}
     */
    stopPullDownRefresh() {
        if (wx.stopPullDownRefresh) {
            return wx.stopPullDownRefresh()
        } else {
            return showVersionTips() // 微信低版本兼容提示
        }
    },

    /* WXML节点信息 */
    /**
     * 返回一个SelectorQuery对象实例。可以在这个实例上使用select等方法选择节点，并使用boundingClientRect等方法选择需要查询的信息
     * @returns {*}
     */
    createSelectorQuery() {
        if (wx.createSelectorQuery) {
            return wx.createSelectorQuery()
        } else {
            return showVersionTips() // 微信低版本兼容提示
        }
    },


    /****** 开放接口 ******/
    /* 转发 */
    /**
     * 显示当前页面的转发按钮
     * @param {Boolean} withShareTicket 是否使用带 shareTicket 的转发详情
     * @returns {*}
     */
    showShareMenu(withShareTicket) {
        if (wx.showShareMenu) {
            return wx.showShareMenu({
                withShareTicket: withShareTicket
            })
        } else {
            return showVersionTips() // 微信低版本兼容提示
        }
    },

    /**
     * 隐藏转发按钮
     * @returns {*}
     */
    hideShareMenu() {
        if (wx.hideShareMenu) {
            return wx.hideShareMenu()
        } else {
            return showVersionTips() // 微信低版本兼容提示
        }
    },

    /**
     * 更新转发属性
     * @param {Boolean} withShareTicket 是否使用带 shareTicket 的转发详情
     * @returns {*}
     */
    updateShareMenu(withShareTicket) {
        if (wx.updateShareMenu) {
            return wx.updateShareMenu({
                withShareTicket: withShareTicket
            })
        } else {
            return showVersionTips() // 微信低版本兼容提示
        }
    },

    /**
     * 获取转发详细信息
     * @param {Boolean} shareTicket
     * @param {Number} timeout
     * @returns {*}
     */
    getShareInfo(shareTicket, timeout = 20000) {
        if (wx.getShareInfo) {
            return wx.getShareInfo({
                shareTicket: shareTicket,
                timeout: timeout
            })
        } else {
            return showVersionTips() // 微信低版本兼容提示
        }
    },
}

